﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Core;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;

namespace HeuristicLab.Analysis.SolutionCaching.Views {
  [View("SolutionCache View")]
  [Content(typeof(SolutionCache<,>), true)]
  public abstract partial class SolutionCacheView<TKey, TValue> : ItemView
    where TKey : Item
    where TValue : SolutionInformation<TKey> {
    public SolutionCacheView() {
      InitializeComponent();
    }

    public new SolutionCache<TKey, TValue> Content {
      get { return (SolutionCache<TKey, TValue>)base.Content; }
      set { base.Content = value; }
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      cacheSizeTextBox.Clear();

      if (Content != null) {
        cacheSizeTextBox.Text = Content.Size().ToString();
        InitializeMenu();
      }
    }

    protected virtual void InitializeMenu() {
      var viewTypes = MainFormManager.GetViewTypes(this.Content.GetType(), false);
      foreach (Type viewType in viewTypes.OrderBy(x => ViewAttribute.GetViewName(x))) {
        if (viewType != typeof(ViewHost)) {
          ToolStripMenuItem menuItem = new ToolStripMenuItem();
          menuItem.Text = ViewAttribute.GetViewName(viewType);
          menuItem.Tag = viewType;
          menuItem.Click += new EventHandler(menuItem_Click);
          analyzeToolStripButton.DropDownItems.Add(menuItem);
        }
      }
    }

    protected void menuItem_Click(object sender, EventArgs e) {
      ToolStripMenuItem menuItem = (ToolStripMenuItem)sender;
      Type viewType = (Type)menuItem.Tag;
      IContentView view = MainFormManager.MainForm.ShowContent(Content, viewType);
      if (view != null) {
        view.Locked = Locked;
        view.ReadOnly = ReadOnly;
      }
    }
  }
}
